import TileLayer from "ol/layer/Tile";
import OSM from "ol/source/OSM";
import XYZ from "ol/source/XYZ";
import TileGrid from "ol/tilegrid/TileGrid";
import WMTSGrid from "ol/tilegrid/WMTS";
import WMTS from "ol/source/WMTS";
import TileImage from "ol/source/TileImage";
import { get as getProj } from "ol/proj";
import Projection from "ol/proj/Projection";
import { getWidth, getTopLeft } from "ol/extent";
import {
  validate,
  validateParas,
  get as getConfig,
  getConfigParas
} from "./setting";

// 创建百度地图图层
function createBaiduLayer(config) {
  const projection = getProj("EPSG:3857");
  const tileGrid = new TileGrid({
    origin: [0, 0],
    resolutions: [
      0,
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9,
      10,
      11,
      12,
      13,
      14,
      15,
      16,
      17,
      18
    ].map(function(x) {
      return Math.pow(2, 18 - x);
    })
  });
  let source = new TileImage({
    projection: projection,
    tileGrid: tileGrid,
    tileUrlFunction: function(tileCoord) {
      const z = tileCoord[0];
      const x = tileCoord[1];
      const y = tileCoord[2];
      const index = Math.ceil(Math.random() * 5);
      return config.server
        .replace(/{x}/g, x)
        .replace(/{y}/g, y)
        .replace(/{z}/g, z)
        .replace(/{index}/g, index);
    }
  });
  return new TileLayer({
    source: source
  });
}
// 创建方正地图图层
function createFounderLayer(config) {
  const tileSizePixels = 256;
  const projection = getProj("EPSG:4326");
  const projectionExtent = projection.getExtent();
  const size = getWidth(projectionExtent) / tileSizePixels;
  let matrixIds = [];
  let resolutions = [];
  for (let i = 0; i <= 20; i++) {
    matrixIds[i] = i;
    resolutions[i] = size / Math.pow(2, i);
  }
  const tileGrid = new WMTSGrid({
    origin: getTopLeft(projectionExtent),
    resolutions: resolutions,
    matrixIds: matrixIds
  });
  const basePath = config.server;
  return new TileLayer({
    source: new WMTS({
      url: basePath,
      tileGrid: tileGrid,
      projection: projection,
      tileLoadFunction: function(imageTile) {
        //tag:需要定义一个参数src?
        const tileCoord = imageTile.tileCoord;
        let zoom = tileCoord[0];
        let col = tileCoord[1];
        let row = -(tileCoord[2] + 1);
        imageTile.getImage().src = `${basePath}?Service=getImage&Type=RGB&ZoomOffset=0&Col=${col}&Row=${row}&Zoom=${zoom}&V=0.3`;
      }
    }),
    wrapX: false
  });
}

//创建广州geoserver发布的tms服务
function createGeoTMS(config) {
  console.log(config);
}
//创建广州geoserver发布的wmts服务
function createGeoWMTS(config) {
  console.log(config);
  let matrixIds = [
    "EPSG:4326:0",
    "EPSG:4326:1",
    "EPSG:4326:2",
    "EPSG:4326:3",
    "EPSG:4326:4",
    "EPSG:4326:5",
    "EPSG:4326:6",
    "EPSG:4326:7",
    "EPSG:4326:8",
    "EPSG:4326:9",
    "EPSG:4326:10",
    "EPSG:4326:11",
    "EPSG:4326:12",
    "EPSG:4326:13",
    "EPSG:4326:14",
    "EPSG:4326:15",
    "EPSG:4326:16",
    "EPSG:4326:17",
    "EPSG:4326:18"
  ];

  let resolutions = [
    0.703125,
    0.3515625,
    0.17578125,
    0.087890625,
    0.0439453125,
    0.02197265625,
    0.010986328125,
    0.0054931640625,
    0.00274658203125,
    0.001373291015625,
    6.866455078125e-4,
    3.4332275390625e-4,
    1.71661376953125e-4,
    8.58306884765625e-5,
    4.291534423828125e-5,
    2.1457672119140625e-5,
    1.0728836059570312e-5,
    5.364418029785156e-6,
    2.682209014892578e-6
  ];

  let tilegrid = new WMTSGrid({
    extent: [-180.0, -90.0, 180.0, 90.0],
    tileSize: [256, 256],
    origin: [-180.0, 90.0],
    resolutions: resolutions,
    matrixIds: matrixIds
  });

  let projection = new Projection({
    code: "EPSG:4326",
    units: "degrees",
    axisOrientation: "neu"
  });
  //广州给的wmts服务的图层
  let tileSource = new WMTS({
    url: "http://218.19.252.203:20052/geoserver/gwc/service/wmts",
    layer: "chanzhou:jichutu",
    format: "image/png",
    matrixSet: "EPSG:4326",
    projection: projection,
    tileGrid: tilegrid
  });
  let tileLayer = new TileLayer({
    opacity: 0.7,
    source: tileSource,
    wrapX: false
  });
  return tileLayer;
}
//创建arcgisServer的服务图层
function createArcgisServerWMTS(config) {
  console.log(config);
}
//创建天地图的矢量图层(包括注记)
function createTDTVectorLayer(config) {
  //let center = transform([112.73, 38.42], "EPSG:4326", "EPSG:3857");
  console.log(config);
  let layers = [
    new TileLayer({
      source: new XYZ({
        url:
          "https://t0.tianditu.gov.cn/DataServer?T=vec_w&x={x}&y={y}&l={z}&tk=50f8a772a2efdcc689e9255e9a5e502b"
      }),
      isGroup: true,
      name: "天地图路网"
    }),
    new TileLayer({
      source: new XYZ({
        url:
          "https://t0.tianditu.gov.cn/DataServer?T=cia_w&x={x}&y={y}&l={z}&tk=50f8a772a2efdcc689e9255e9a5e502b"
      }),
      isGroup: true,
      name: "天地图文字标注"
    })
  ];
  return layers[0];
}
//创建天地图的图层(包括注记)
function createTDTImageLayer(config) {
  //let center = transform([112.73, 38.42], "EPSG:4326", "EPSG:3857");
  console.log(config);
  let layers = [
    new TileLayer({
      source: new XYZ({
        url:
          "https://t0.tianditu.gov.cn/DataServer?T=img_w&x={x}&y={y}&l={z}&tk=50f8a772a2efdcc689e9255e9a5e502b"
      }),
      title: "卫星图",
      isGroup: true
    }),
    new TileLayer({
      source: new XYZ({
        url:
          "https://t0.tianditu.gov.cn/DataServer?T=cia_w&x={x}&y={y}&l={z}&tk=50f8a772a2efdcc689e9255e9a5e502b"
      }),
      isGroup: true,
      name: "天地图文字标注"
    })
  ];
  return layers[0];
}

export function createLayer(key = "OSM") {
  if (!validate(key)) {
    return null;
  }
  const config = getConfig(key);
  let layer = null;
  switch (key) {
    case "OSM":
      layer = new TileLayer({
        source: new OSM()
      });
      break;
    // Google和高德的方式相同
    case "Google":
    case "Amap":
      layer = new TileLayer({
        source: new XYZ({
          url: config.server
        })
      });
      break;
    case "Baidu":
      layer = createBaiduLayer(config);
      break;
    case "Founder":
      layer = createFounderLayer(config);
      break;
    case "TDTVector":
      layer = createTDTVectorLayer(config);
      break;
    case "TDTImage":
      layer = createTDTImageLayer(config);
      break;
    case "ArcgisServerWMTS":
      layer = createArcgisServerWMTS(config);
      break;
    case "GeoWMTS":
      layer = createGeoWMTS(config);
      break;
    case "GeoTMS":
      layer = createGeoTMS(config);
  }
  return layer;
}

export function createLayers(paras = [{ type: "OSM", URL: "" }]) {
  if (!validateParas(paras)) {
    return null;
  }
  const configParas = getConfigParas(paras);
  let layers = [];
  console.log(configParas + layers);
}
